if not LibDataBroker then LibDataBroker = {} end -- must be global because there is no LibStub
local lib = LibDataBroker -- make local reference

lib.attributestorage, lib.namestorage, lib.proxystorage = lib.attributestorage or {}, lib.namestorage or {}, lib.proxystorage or {}
local attributestorage, namestorage = lib.attributestorage, lib.namestorage

-- Rift conversion to callbacks stuff
lib.FireAttributeChanged = lib.FireAttributeChanged or Utility.Event.Create("LibDataBroker", "AttributeChanged")
lib.FireObjectCreated = lib.FireObjectCreated or Utility.Event.Create("LibDataBroker", "ObjectCreated")

lib.domt = {
	__metatable = "access denied",
	__index = function(self, key) return attributestorage[self] and attributestorage[self][key] end,
}

lib.domt.__newindex = function(self, key, value)
	if not attributestorage[self] then attributestorage[self] = {} end
	if attributestorage[self][key] == value then return end
	attributestorage[self][key] = value
	local name = namestorage[self]
	if not name then return end
		
	--[[
	callbacks:Fire("LibDataBroker_AttributeChanged", name, key, value, self)
	callbacks:Fire("LibDataBroker_AttributeChanged_"..name, name, key, value, self)
	callbacks:Fire("LibDataBroker_AttributeChanged_"..name.."_"..key, name, key, value, self)
	callbacks:Fire("LibDataBroker_AttributeChanged__"..key, name, key, value, self)
	]]--
	
	-- Rift conversion code
	lib.FireAttributeChanged(name, key, value, self)
end

function lib:NewDataObject(name, dataobj)
	if self.proxystorage[name] then return end

	if dataobj then
		assert(type(dataobj) == "table", "Invalid dataobj, must be nil or a table")
		self.attributestorage[dataobj] = {}
		for i,v in pairs(dataobj) do
			self.attributestorage[dataobj][i] = v
			dataobj[i] = nil
		end
	end
	dataobj = setmetatable(dataobj or {}, self.domt)
	self.proxystorage[name], self.namestorage[dataobj] = dataobj, name
	-- self.callbacks:Fire("LibDataBroker_DataObjectCreated", name, dataobj)
	
	-- Rift conversion code
	lib.FireObjectCreated(name, dataobj)
	
	return dataobj
end

function lib:DataObjectIterator()
	return pairs(self.proxystorage)
end

function lib:GetDataObjectByName(dataobjectname)
	return self.proxystorage[dataobjectname]
end

function lib:GetNameByDataObject(dataobject)
	return self.namestorage[dataobject]
end

local next = pairs(attributestorage)
function lib:pairs(dataobject_or_name)
	local t = type(dataobject_or_name)
	assert(t == "string" or t == "table", "Usage: ldb:pairs('dataobjectname') or ldb:pairs(dataobject)")

	local dataobj = self.proxystorage[dataobject_or_name] or dataobject_or_name
	assert(attributestorage[dataobj], "Data object not found")

	return next, attributestorage[dataobj], nil
end

local ipairs_iter = ipairs(attributestorage)
function lib:ipairs(dataobject_or_name)
	local t = type(dataobject_or_name)
	assert(t == "string" or t == "table", "Usage: ldb:ipairs('dataobjectname') or ldb:ipairs(dataobject)")

	local dataobj = self.proxystorage[dataobject_or_name] or dataobject_or_name
	assert(attributestorage[dataobj], "Data object not found")

	return ipairs_iter, attributestorage[dataobj], 0
end