-- Neglected's FooBar AddOn.
if FooBar then return; end -- Do not load if we already have FooBar loaded in the global environment
local LDB = LibDataBroker; -- Locallized index for pew pewing.

FooBar = {
	mContext = UI.CreateContext("FooBarContext"),
	getFoobar = function() return FooBar.mMainFrame; end,
	FooBarModuleLayer = 0,
	FooBarModuleXPadding = 2,
	FooBarModuleYPadding = 2,
	FooBarGlobalTextSize = 17,
	
	
	-- misc options
	allowHeightResizeFromTextChange = true, -- allows the height of the bar to change if a module sets the height of it's text
											-- larger than bar height - padding.
};
-- initialize these outside of the table to allow access to FooBar.mContext.
FooBar.mMainFrame = UI.CreateFrame("Frame", "FooBarFrame", FooBar.mContext);
FooBar.mTextureFrame = UI.CreateFrame("Texture", "FooBarTextureFrame", FooBar.mMainFrame);
FooBar.FooBarModuleLayer = FooBar.mMainFrame:GetLayer() + 1;

-- simple function that gets our screen res.
local function getScreenResolution()
	return UIParent:GetWidth(), UIParent:GetHeight();
end

-- This is fired when the height of any module is changed.
-- It (in theory) should iterate through all modules and then change their y orientation so they are dead center on the Y axis
-- and make sure that all modules are x padding apart.
function FooBar:OnDimensionsChanged()
	-- iterate through all modules.
	-- change height of texture frame
	FooBar.mTextureFrame:SetHeight(FooBar.mMainFrame:GetHeight());
	local x_point = self.FooBarModuleXPadding; -- this is the initial x point for the first module
	local y_point = (self:getFoobar():GetHeight() / 2); -- hopefully so that the module is dead center.
	for i = 1, FooBarModule.mNumModules do
		if FooBarModule.modules[i-1] then
			x_point = x_point + FooBarModule.modules[i-1]:getWidth();
		end
		FooBarModule.modules[i]:setPoint("CENTER", self.mContext, "CENTER", x_point, y_point);
	end

end

function FooBar:getXPadding() return self.FooBarModuleXPadding; end
function FooBar:getYPadding() return self.FooBarModuleYPadding; end

-- Our LibDataBroker callbacks.
-- See, the issue I have with this is that there's no real way us to know what values to display on the FooBar.
-- I'm not really sure what to do here, if I'm honest :$
-- I'll ask Myr later on, but I'm just leaving this here for proof-of-concept "hai we haev supp0rtkai?".
function FooBar:OnLDBAttributeChanged(name, key, value, dataObject)
	-- Here we should probably change the name (if it is changed).
	-- Again, there's no real way to know what to display in the way of changes unless
	-- we enforce that dataobjects have certain functions in them, like "display".
	-- or display = { firstElement , secondElement }
	-- So we can then draw those.
end
function FooBar:OnLDBObjectCreated(name, dataObject)
	-- Put the new object onto the foobar
	newModule = FooBarModule:new(name);
	newModule:setText(name);
end
table.insert(Event.LibDataBroker.AttributeChanged, {FooBar.OnLDBAttributeChanged, "FooBar", "FooBar_LDB_AttribChanged"});
table.insert(Event.LibDataBroker.ObjectCreated, {FooBar.OnLDBObjectCreated, "FooBar", "FooBar_LDB_ObjectCreated"});


-- Create our main frame.
do
	local res_x, res_y = getScreenResolution();
	
	FooBar.mMainFrame:SetVisible(true);
	FooBar.mMainFrame:SetWidth(res_x);
	FooBar.mMainFrame:SetAlpha(1);
	FooBar.mMainFrame:SetBackgroundColor(0, 0, 0);
	FooBar.mMainFrame:SetHeight(20);
	FooBar.mMainFrame:SetLayer(2);
end